custom.spline <- function (x, df = NULL, knots = NULL, intercept = FALSE, Boundary.knots = range(x),  
                           left.outer=NULL, right.outer=NULL, ord=4L, left.extra=1, right.extra=1, restrictions=c(2,2)){
  x <- as.vector(x)
  nax <- is.na(x)
  if (nas <- any(nax)) 
    x <- x[!nax]
  if (!missing(Boundary.knots)) {
    Boundary.knots <- sort(Boundary.knots)
    outside <- (ol <- x < Boundary.knots[1L]) | (or <- x > 
                                                   Boundary.knots[2L])
  } else outside <- FALSE
  if (!is.null(df) && is.null(knots)) {
    nIknots <- df -ord+length(restrictions) - 1L - intercept
    #nIknots <- df - 1L - intercept
    if (nIknots < 0L) {
      nIknots <- 0L
      warning(gettextf("'df' was too small; have used %d", 
                       1L + intercept), domain = NA)
    }
    knots <- if (nIknots > 0L) {
      knots <- seq.int(0, 1, length.out = nIknots + 2L)[-c(1L, 
                                                           nIknots + 2L)]
      stats::quantile(x[!outside], knots)
    }
  }
  else nIknots <- length(knots)
  if (is.null(left.outer) & is.null(right.outer)){
   Aknots <- sort(c(rep(Boundary.knots, ord), knots))
  }else{
    Aknots <- sort(c(left.outer, knots,right.outer))
  } 
  if (any(outside)) {
    basis <- array(0, c(length(x), nIknots + ord))
    if (any(ol)) {
      k.pivot <- Boundary.knots[1L]
      if(left.extra==0){
        basis[ol, ] <- rep(splineDesign(Aknots, k.pivot, ord),each=sum(ol))
      }else{
        basis[ol, ] <- xl %*% tt
        xl <- cbind(1, x[ol] - k.pivot)
        #use linear extrapolation
        tt <- splineDesign(Aknots, rep(k.pivot, left.extra+1L), ord, 
                           seq_len(left.extra+1L)-1L)
        basis[ol, ] <- xl %*% tt
      }
    }
    if (any(or)) {
      k.pivot <- Boundary.knots[2L]
      if(right.extra==0){
        basis[or, ] <- rep(splineDesign(Aknots, k.pivot, ord),each=sum(or))
      }else{
        xr <- cbind(1, x[or] - k.pivot)
        # linear extrapolation
        tt <- splineDesign(Aknots, rep(k.pivot, right.extra+1L), ord, 
                           seq_len(right.extra+1L)-1L)
        basis[or, ] <- xr %*% tt
      }
    }
    if (any(inside <- !outside)) 
      basis[inside, ] <- splineDesign(Aknots, x[inside], 
                                      ord)
    
  }
  else basis <- splineDesign(Aknots, x, ord)
  # Now place restrictions
  if (NROW(restrictions>0L)) {
    const <- splineDesign(Aknots, restrictions[,1], ord, restrictions[,2])
  }  
  if (!intercept) {
    if (NROW(restrictions>0L)) {
     const <- const[, -1, drop = FALSE]
    }  
    basis <- basis[, -1, drop = FALSE]
  }
  if (NROW(restrictions>0L)) {
    qr.const <- qr(t(const))
    basis <- as.matrix((t(qr.qty(qr.const, t(basis))))[ ,-seq.int(NROW(restrictions)),drop=FALSE]) 
  } 
  n.col <- ncol(basis)
  if (nas) {
    nmat <- matrix(NA, length(nax), n.col)
    nmat[!nax, ] <- basis
    basis <- nmat
  }
  
  a <- list(degree = 3L, knots = if (is.null(knots)) numeric() else knots, 
            Boundary.knots = Boundary.knots, intercept = intercept)
  attributes(basis) <- c(attributes(basis), a)
  class(basis) <- c("ns", "basis", "matrix")
  basis
}